@@ -439,6 +439,7 @@ void forestry::addTrees(size_t ntree) {
 std::unique_ptr< std::vector<double> > forestry::predict(
   std::vector< std::vector<double> >* xNew,
   arma::Mat<double>* weightMatrix,
+  arma::Mat<double>* coefficients,
   arma::Mat<int>* terminalNodes,
   unsigned int seed,
   size_t nthread,
@@ -452,6 +453,21 @@ std::unique_ptr< std::vector<double> > forestry::predict(
     prediction.push_back(0);
   }
 
+  // If we want to return the ridge coefficients, initialize a matrix
+  if (coefficients) {
+    // Create coefficient vector of vectors of zeros
+    std::vector< std::vector<float> > coef;
+    size_t numObservations = (*xNew)[0].size();
+    size_t numCol = (*coefficients).n_cols;
+    for (size_t i=0; i<numObservations; i++) {
+      std::vector<float> row;
+      for (size_t j = 0; j<numCol; j++) {
+        row.push_back(0);
+      }
+      coef.push_back(row);
+    }
+  }
+
   // Only needed if exact = TRUE, vector for storing each tree's predictions
   std::vector< std::vector<double> > tree_preds;
   std::vector< std::vector<int> > tree_nodes;
@@ -490,20 +506,42 @@ std::unique_ptr< std::vector<double> > forestry::predict(
           try {
             std::vector<double> currentTreePrediction(numObservations);
             std::vector<int> currentTreeTerminalNodes(numObservations);
+            std::vector< std::vector<double> > currentTreeCoefficients(numObservations);
 
             //If terminal nodes, pass option to tree predict
             forestryTree *currentTree = (*getForest())[i].get();
-            (*currentTree).predict(
-              currentTreePrediction,
-              &currentTreeTerminalNodes,
-              xNew,
-              getTrainingData(),
-              weightMatrix,
-              getlinear(),
-              seed + i,
-              getMinNodeSizeToSplitAvg()
-            );
 
+            if (coefficients) {
+              for (size_t l=0; l<numObservations; l++) {
+                currentTreeCoefficients[l] = std::vector<double>(coefficients->n_cols);
+              }
+
+              (*currentTree).predict(
+                  currentTreePrediction,
+                  &currentTreeTerminalNodes,
+                  currentTreeCoefficients,
+                  xNew,
+                  getTrainingData(),
+                  weightMatrix,
+                  getlinear(),
+                  seed + i,
+                  getMinNodeSizeToSplitAvg()
+              );
+
+            } else {
+              (*currentTree).predict(
+                  currentTreePrediction,
+                  &currentTreeTerminalNodes,
+                  currentTreeCoefficients,
+                  xNew,
+                  getTrainingData(),
+                  weightMatrix,
+                  getlinear(),
+                  seed + i,
+                  getMinNodeSizeToSplitAvg()
+              );
+
+            }
 
             // HERE IF NEED TERMINAL NODES, pass option to tree predict, then
             // lock thread (shouldn't really need to), use i as offset and flip
@@ -515,16 +553,29 @@ std::unique_ptr< std::vector<double> > forestry::predict(
 
             // If we need to use the exact seeding order we save the tree
             // predictions and the tree seeds
+
+            // For now store tree seeds even when not running exact,
+            // hopefully this solves a valgrind error relating to the sorting
+            // based on tree seeds when tree seeds might be uninitialized
+            tree_seeds.push_back(currentTree->getSeed());
+
             if (exact) {
               tree_preds.push_back(currentTreePrediction);
               tree_nodes.push_back(currentTreeTerminalNodes);
-              tree_seeds.push_back(currentTree->getSeed());
               tree_total_nodes.push_back(currentTree->getNodeCount());
             } else {
               for (size_t j = 0; j < numObservations; j++) {
                 prediction[j] += currentTreePrediction[j];
               }
 
+              if (coefficients) {
+                for (size_t k = 0; k < numObservations; k++) {
+                  for (size_t l = 0; l < coefficients->n_cols; l++) {
+                    (*coefficients)(k,l) += currentTreeCoefficients[k][l];
+                  }
+                }
+              }
+
               if (terminalNodes) {
                 for (size_t k = 0; k < numObservations; k++) {
                   (*terminalNodes)(k, i) = currentTreeTerminalNodes[k];
@@ -561,7 +612,7 @@ std::unique_ptr< std::vector<double> > forestry::predict(
   if (exact) {
     std::vector<size_t> indices(tree_seeds.size());
     std::iota(indices.begin(), indices.end(), 0);
-    // Order the indices by the seeds of the corresponding trees
+    //Order the indices by the seeds of the corresponding trees
     std::sort(indices.begin(), indices.end(),
               [&](size_t a, size_t b) -> bool {
                 return tree_seeds[a] > tree_seeds[b];
@@ -600,7 +651,6 @@ std::unique_ptr< std::vector<double> > forestry::predict(
     prediction[j] /= total_weights;
   }
 
-
   std::unique_ptr< std::vector<double> > prediction_ (
     new std::vector<double>(prediction)
   );
@@ -618,6 +668,14 @@ std::unique_ptr< std::vector<double> > forestry::predict(
     }
   }
 
+  if (coefficients) {
+    for (size_t k = 0; k < numObservations; k++) {
+      for (size_t l = 0; l < coefficients->n_cols; l++) {
+        (*coefficients)(k,l) /= total_weights;
+      }
+    }
+  }
+
   return prediction_;
 }
 
@@ -977,6 +1035,7 @@ void forestry::fillinTreeInfo(
 
 void forestry::reconstructTrees(
     std::unique_ptr< std::vector<size_t> > & categoricalFeatureColsRcpp,
+    std::unique_ptr< std::vector<unsigned int> > & tree_seeds,
     std::unique_ptr< std::vector< std::vector<int> >  > & var_ids,
     std::unique_ptr< std::vector< std::vector<double> >  > & split_vals,
     std::unique_ptr< std::vector< std::vector<int> >  > & naLeftCounts,
@@ -1004,6 +1063,7 @@ void forestry::reconstructTrees(
                 gethasNas(),
                 getlinear(),
                 getOverfitPenalty(),
+                (*tree_seeds)[i],
                 (*categoricalFeatureColsRcpp),
                 (*var_ids)[i],
                 (*split_vals)[i],
