@@ -467,6 +467,7 @@ Rcpp::List rcpp_cppPredictInterface(
     // then we inialize the empty weight matrix
     arma::Mat<double> weightMatrix;
     arma::Mat<int> terminalNodes;
+    arma::Mat<double> coefficients;
 
     // Have to keep track of tree_weights
     std::vector<size_t>* testForestTreeWeights;
@@ -485,7 +486,25 @@ Rcpp::List rcpp_cppPredictInterface(
       threads_to_use = (size_t) nthread;
     }
 
-    if (aggregation == "weightMatrix") {
+    if (aggregation == "coefs") {
+      size_t nrow = featureData[0].size();
+      // Now we need the number of linear features + 1 for the intercept
+      size_t ncol = (*testFullForest).getTrainingData()->getLinObsData(0).size() + 1;
+      //Set coefficients to be zero
+      coefficients.resize(nrow, ncol);
+      coefficients.zeros(nrow, ncol);
+
+      testForestPrediction = (*testFullForest).predict(&featureData,
+                                                       NULL,
+                                                       &coefficients,
+                                                       NULL,
+                                                       seed,
+                                                       threads_to_use,
+                                                       false,
+                                                       false,
+                                                       NULL);
+
+    } else if (aggregation == "weightMatrix") {
       size_t nrow = featureData[0].size(); // number of features to be predicted
       size_t ncol = (*testFullForest).getNtrain(); // number of train data
       weightMatrix.resize(nrow, ncol); // initialize the space for the matrix
@@ -497,6 +516,7 @@ Rcpp::List rcpp_cppPredictInterface(
       testForestPrediction = (*testFullForest).predict(&featureData,
                                                        &weightMatrix,
                                                        NULL,
+                                                       NULL,
                                                        seed,
                                                        threads_to_use,
                                                        exact,
@@ -522,15 +542,17 @@ Rcpp::List rcpp_cppPredictInterface(
       // The idea is that, if the weightMatrix is point to NULL it won't be
       // be updated, but otherwise it will be updated:
       testForestPrediction = (*testFullForest).predict(&featureData,
-                              &weightMatrix,
-                              &terminalNodes,
-                              seed,
-                              threads_to_use,
-                              exact,
-                              false,
-                              NULL);
+                                                       &weightMatrix,
+                                                       NULL,
+                                                       &terminalNodes,
+                                                       seed,
+                                                       threads_to_use,
+                                                       exact,
+                                                       false,
+                                                       NULL);
     } else {
       testForestPrediction = (*testFullForest).predict(&featureData,
+                                                       NULL,
                                                        NULL,
                                                        NULL,
                                                        seed,
@@ -545,9 +567,13 @@ Rcpp::List rcpp_cppPredictInterface(
 
     Rcpp::NumericVector predictions = Rcpp::wrap(*testForestPrediction_);
 
+    delete testForestPrediction_;
+    delete testForestTreeWeights;
+
     return Rcpp::List::create(Rcpp::Named("predictions") = predictions,
                               Rcpp::Named("weightMatrix") = weightMatrix,
-                              Rcpp::Named("terminalNodes") = terminalNodes);
+                              Rcpp::Named("terminalNodes") = terminalNodes,
+                              Rcpp::Named("coef") = coefficients);
 
     // return output;
 
@@ -753,6 +779,7 @@ Rcpp::List rcpp_CppToR_translator(
 
     // Return the lis of list. For each tree an element in the first list:
     Rcpp::List list_to_return;
+
     for(size_t i=0; i!=forest_dta->size(); i++){
       Rcpp::IntegerVector var_id = Rcpp::wrap(((*forest_dta)[i]).var_id);
 
@@ -804,7 +831,8 @@ Rcpp::List rcpp_CppToR_translator(
 			   Rcpp::Named("averagingSampleIndex") = averagingSampleIndex,
 			   Rcpp::Named("splittingSampleIndex") = splittingSampleIndex,
 			   Rcpp::Named("naLeftCounts") = naLeftCounts,
-			   Rcpp::Named("naRightCounts") = naRightCounts
+			   Rcpp::Named("naRightCounts") = naRightCounts,
+			   Rcpp::Named("seed") = (*forest_dta)[i].seed // Add the seeds to the list we return
         );
 
       // std::cout << "finished list\n";
@@ -816,6 +844,8 @@ Rcpp::List rcpp_CppToR_translator(
     // std::cout << "hello1\n";
     // std::cout.flush();
 
+
+
     return list_to_return;
 
   } catch(std::runtime_error const& err) {
@@ -943,7 +973,8 @@ Rcpp::List rcpp_multilayer_CppToR_translator(
             Rcpp::Named("averagingSampleIndex") = averagingSampleIndex,
             Rcpp::Named("splittingSampleIndex") = splittingSampleIndex,
             Rcpp::Named("naLeftCounts") = naLeftCounts,
-            Rcpp::Named("naRightCounts") = naRightCounts
+            Rcpp::Named("naRightCounts") = naRightCounts,
+            Rcpp::Named("seed") = (*(forest_dta[j]))[i].seed
           );
 
         // std::cout << "finished list\n";
@@ -954,11 +985,11 @@ Rcpp::List rcpp_multilayer_CppToR_translator(
         // std::cout << i << "pushed list\n";
         // std::cout.flush();
       }
-      list_to_return.push_back(list_to_return_j);
 
       // std::cout << "pushing final list\n";
       // std::cout.flush();
 
+      list_to_return.push_back(list_to_return_j);
     }
 
     // std::cout << "hello1\n";
@@ -1039,6 +1070,9 @@ Rcpp::List rcpp_reconstructree(
   std::unique_ptr< std::vector< std::vector<size_t> > > splittingSampleIndex(
       new  std::vector< std::vector<size_t> >
   );
+  std::unique_ptr< std::vector<unsigned int> > tree_seeds(
+      new std::vector<unsigned int>
+  );
 
 
 
@@ -1068,7 +1102,9 @@ Rcpp::List rcpp_reconstructree(
     naRightCounts->push_back(
         Rcpp::as< std::vector<int> > ((Rcpp::as<Rcpp::List>(R_forest[i]))[7])
     );
-
+    tree_seeds->push_back(
+        Rcpp::as< unsigned int > ((Rcpp::as<Rcpp::List>(R_forest[i]))[8])
+    );
   }
 
   // Decode catCols and R_forest
@@ -1087,11 +1123,6 @@ Rcpp::List rcpp_reconstructree(
     (*categoricalFeatureColsRcpp_copy).push_back(
         (*categoricalFeatureColsRcpp)[i]);
   }
-  (
-      new std::vector<size_t>(
-          Rcpp::as< std::vector<size_t> >(catCols)
-      )
-  ); // contains the col indices of categorical features.
 
   std::unique_ptr<std::vector< std::vector<double> > > featureDataRcpp (
       new std::vector< std::vector<double> >(
@@ -1190,6 +1221,7 @@ Rcpp::List rcpp_reconstructree(
   );
 
   testFullForest->reconstructTrees(categoricalFeatureColsRcpp_copy,
+                                   tree_seeds,
                                    var_ids,
                                    split_vals,
                                    naLeftCounts,
@@ -1263,6 +1295,7 @@ Rcpp::List rcpp_reconstruct_forests(
   std::vector< std::unique_ptr< std::vector< std::vector<size_t> > > > leafSplidxs;
   std::vector< std::unique_ptr< std::vector< std::vector<size_t> > > > averagingSampleIndex;
   std::vector< std::unique_ptr< std::vector< std::vector<size_t> > > > splittingSampleIndex;
+  std::vector< std::unique_ptr< std::vector<unsigned int> > > tree_seeds;
 
   std::vector< forestry* > multilayerForests;
   // Now we need to iterate through the length of number forests, and for each
@@ -1280,6 +1313,7 @@ Rcpp::List rcpp_reconstruct_forests(
       std::vector< std::vector<size_t> > cur_leafSplidxs;
       std::vector< std::vector<size_t> > cur_averagingSampleIndex;
       std::vector< std::vector<size_t> > cur_splittingSampleIndex;
+      std::vector< unsigned int > cur_tree_seeds;
 
     // Now for the current forest, we iterate through and build the trees
     for(size_t i = 0; i < (size_t) Rcpp::as<Rcpp::List>(R_forests[j]).size(); i++){
@@ -1318,6 +1352,9 @@ Rcpp::List rcpp_reconstruct_forests(
         Rcpp::as< std::vector<int> > (Rcpp::as<Rcpp::List>(Rcpp::as<Rcpp::List>(R_forests[j])[i])[7])
       );
 
+      cur_tree_seeds.push_back(
+        Rcpp::as< unsigned int > (Rcpp::as<Rcpp::List>(Rcpp::as<Rcpp::List>(R_forests[j])[i])[8])
+      );
     }
     // Now the cur vectors hold the info for each tree, we have to
     // add this info to the vector of forests
@@ -1353,6 +1390,10 @@ Rcpp::List rcpp_reconstruct_forests(
         new std::vector< std::vector<size_t> >(cur_splittingSampleIndex)
     ));
 
+    tree_seeds.push_back(std::unique_ptr< std::vector<unsigned int> >(
+        new std::vector< unsigned int >(cur_tree_seeds)
+    ));
+
     // Decode catCols and R_forest
     std::unique_ptr< std::vector<size_t> > categoricalFeatureColsRcpp (
         new std::vector<size_t>(
@@ -1369,11 +1410,6 @@ Rcpp::List rcpp_reconstruct_forests(
       (*categoricalFeatureColsRcpp_copy).push_back(
           (*categoricalFeatureColsRcpp)[i]);
     }
-    (
-        new std::vector<size_t>(
-            Rcpp::as< std::vector<size_t> >(catCols)
-        )
-    ); // contains the col indices of categorical features.
 
     std::unique_ptr<std::vector< std::vector<double> > > featureDataRcpp (
         new std::vector< std::vector<double> >(
@@ -1478,6 +1514,7 @@ Rcpp::List rcpp_reconstruct_forests(
     // std::cout.flush();
     // Reconstruct the jth forest with its tree info
     testFullForest->reconstructTrees(categoricalFeatureColsRcpp_copy,
+                                     tree_seeds[j],
                                      var_ids[j],
                                      split_vals[j],
                                      naLeftCounts[j],
@@ -1498,59 +1535,65 @@ Rcpp::List rcpp_reconstruct_forests(
           Rcpp::as< std::vector<size_t> >(catCols)
       )
   ); // contains the col indices of categorical features.
+
   std::unique_ptr< std::vector<size_t> > categoricalFeatureColsRcpp_copy(
       new std::vector<size_t>
   );
+
   for(size_t i=0; i<(*categoricalFeatureColsRcpp).size(); i++){
     (*categoricalFeatureColsRcpp_copy).push_back(
         (*categoricalFeatureColsRcpp)[i]);
   }
-  (
-      new std::vector<size_t>(
-          Rcpp::as< std::vector<size_t> >(catCols)
-      )
-  ); // contains the col indices of categorical features.
+
   std::unique_ptr<std::vector< std::vector<double> > > featureDataRcpp (
       new std::vector< std::vector<double> >(
           Rcpp::as< std::vector< std::vector<double> > >(x)
       )
   );
+
   std::unique_ptr< std::vector<double> > outcomeDataRcpp (
       new std::vector<double>(
           Rcpp::as< std::vector<double> >(y)
       )
   );
+
   std::unique_ptr< std::vector<size_t> > linearFeats (
       new std::vector<size_t>(
           Rcpp::as< std::vector<size_t> >(linCols)
       )
   );
+
   std::sort(linearFeats->begin(), linearFeats->end());
   std::unique_ptr< std::vector<double> > featureWeightsRcpp (
       new std::vector<double>(
           Rcpp::as< std::vector<double> >(featureWeights)
       )
   );
+
   std::unique_ptr< std::vector<size_t> > featureWeightsVariablesRcpp (
       new std::vector<size_t>(
           Rcpp::as< std::vector<size_t> >(featureWeightsVariables)
       )
   );
+
   std::unique_ptr< std::vector<double> > deepFeatureWeightsRcpp (
       new std::vector<double>(
           Rcpp::as< std::vector<double> >(deepFeatureWeights)
       )
   );
+
   std::unique_ptr< std::vector<size_t> > deepFeatureWeightsVariablesRcpp (
       new std::vector<size_t>(
           Rcpp::as< std::vector<size_t> >(deepFeatureWeightsVariables)
       )
   );
+
   std::unique_ptr< std::vector<double> > observationWeightsRcpp (
       new std::vector<double>(
           Rcpp::as< std::vector<double> >(observationWeights)
       )
   );
+
   std::unique_ptr< std::vector<int> > monotonicConstraintsRcpp (
       new std::vector<int>(
           Rcpp::as< std::vector<int> >(monotonicConstraints)
@@ -1650,6 +1693,7 @@ std::vector< std::vector<double> > rcpp_cppImputeInterface(
   testForestPrediction = (*testFullForest).predict(&featureData,
                                                    &weightMatrix,
                                                    NULL,
+                                                   NULL,
                                                    seed,
                                                    testFullForest->getNthread(),
                                                    false,
